from typing import List, Optional

from sampledbapi import SampleDBObject, getData

__all__ = ["Location", "getList", "get"]


class Location(SampleDBObject):

    location_id: Optional[int] = None
    name: Optional[str] = None
    description: Optional[str] = None
    parent_location_id: Optional[int] = None
    type_id: Optional[int] = None
    is_hidden: Optional[bool] = None

    def __repr__(self) -> str:
        return f"Location {self.location_id} ({self.name})"


def getList() -> List[Location]:
    """Get a list of all locations.

    Returns:
        List: List of :class:`~sampledbapi.locations.Location` objects.
    """
    return [Location(loc) for loc in getData("locations")]


def get(location_id: int) -> Location:
    """Get the specific location (location_id).

    Args:
        location_id (int): ID of the location to be retrieved.

    Returns:
        Location: The requested :class:`~sampledbapi.locations.Location`.
    """
    if isinstance(location_id, int):
        return Location(getData(f"locations/{location_id}"))
    else:
        raise TypeError()



from typing import List, Optional

from sampledbapi import SampleDBObject, getData

__all__ = ["LocationType", "getList", "get"]


class LocationType(SampleDBObject):

    location_type_id: Optional[int] = None
    name: Optional[str] = None

    def __repr__(self) -> str:
        return f"LocationType {self.location_type_id} ({self.name})"


def getList() -> List[LocationType]:
    """Get a list of all location types.

    Returns:
        List: List of :class:`~sampledbapi.locationtypes.LocationType` objects.
    """
    return [LocationType(loct) for loct in getData("location_types")]


def get(location_type_id: int) -> LocationType:
    """Get the specific location type (location_type_id).

    Args:
        location_type_id (int): ID of the location to be retrieved.

    Returns:
        Location: The requested :class:`~sampledbapi.locationtypes.LocationType`.
    """
    if isinstance(location_type_id, int):
        return LocationType(getData(f"location_types/{location_type_id}"))
    else:
        raise TypeError()



import pytest
from sampledbapi import objects, authenticate
from test import test_actions, test_objects

def mock_authenticate(requests_mock):
    requests_mock.get("http://128.176.208.107:8000/api/v1/actions", text=test_actions.mock_actions())
    requests_mock.get("http://128.176.208.107:8000/api/v1/objects", text=test_objects.mock_objects())
    requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1", text=test_objects.mock_object())
    requests_mock.post("http://128.176.208.107:8000/api/v1/objects/", text=test_objects.mock_objects())
    requests_mock.put("http://128.176.208.107:8000/api/v1/objects/1/permissions/public")
    server_address = "http://128.176.208.107:8000"
    api_key = "Success"
    authenticate(server_address, api_key)

class TestAuthentication():
    
    def test_AuthenticationRequired_get(self):
        with pytest.raises(Exception):
            authenticate(None, None)
        with pytest.raises(Exception):
            objects.getList()
            
    def test_AuthenticationRequired_post(self):
        with pytest.raises(Exception):
            authenticate(None, None)
        with pytest.raises(Exception):
            objects.create(1, {'name': 'Test'})
            
    def test_AuthenticationRequired_put(self, requests_mock):
        obj = objects.Object({})
        with pytest.raises(Exception):
            authenticate(None, None)
        with pytest.raises(Exception):
            obj.setPublic(False) 
        
    def test_authenticate_fail(self, requests_mock):
        requests_mock.get("http://128.176.208.107:8000/api/v1/actions", status_code=404)
        with pytest.raises(Exception):
            server_address = "http://128.176.208.107:8000"
            api_key = None
            authenticate(server_address, api_key)
        with pytest.raises(Exception):
            api_key = "Failure"
            authenticate(server_address, api_key)
        
    def test_authenticate_success(self, requests_mock):
        requests_mock.get("http://128.176.208.107:8000/api/v1/actions", text=test_actions.mock_actions())
        server_address = "http://128.176.208.107:8000"
        api_key = "Success"
        authenticate(server_address, api_key)
        
    def test_json_fail(self, requests_mock):
        requests_mock.get("http://128.176.208.107:8000/api/v1/actions", text="Fail")
        server_address = "http://128.176.208.107:8000"
        api_key = "Success"
        with pytest.raises(Exception):
            authenticate(server_address, api_key)


from typing import List, Optional

from sampledbapi import SampleDBObject, getData

__all__ = ["User", "getList", "get", "getCurrent"]


class User(SampleDBObject):

    user_id: Optional[int] = None
    name: Optional[str] = None
    orcid: Optional[str] = None
    affiliation: Optional[str] = None

    def __repr__(self) -> str:
        return f"User {self.user_id} ({self.name})"


def getList() -> List[User]:
    """Get a list of all users.

    Returns:
        List: List of :class:`~sampledbapi.users.User` objects.
    """
    return [User(d) for d in getData("users")]


def get(user_id: int) -> User:
    """Get the specific user (user_id).

    Args:
        user_id (int): ID of the user to be retrieved.

    Returns:
        User: Requested :class:`~sampledbapi.users.User`.
    """
    if isinstance(user_id, int):
        return User(getData(f"users/{user_id}"))
    else:
        raise TypeError()


def getCurrent() -> User:
    """Get the current user.

    Returns:
        User: Current :class:`~sampledbapi.users.User`.
    """
    return User(getData("users/me"))



import pytest
from sampledbapi import actiontypes
from test import test_authentication

def mock_actiontype():
    return '''{
            "type_id": 1,
            "name": "Create Sample",
            "object_name": "Sample",
            "admin_only": false
        }'''

def mock_actiontypes():
    return f'[{mock_actiontype()},{mock_actiontype()},{mock_actiontype()}]'

class TestActiontypes():
    
    @pytest.fixture(autouse=True)
    def test_init(self, requests_mock):
        requests_mock.get("http://128.176.208.107:8000/api/v1/action_types", text=mock_actiontypes())
        requests_mock.get("http://128.176.208.107:8000/api/v1/action_types/1", text=mock_actiontype())
        test_authentication.mock_authenticate(requests_mock)
        
    def test_getList(self, requests_mock):
        acttyps = actiontypes.getList()
        assert len(acttyps) == 3
        
    def test_get_fail(self, requests_mock):
        with pytest.raises(TypeError):
            actiontypes.get('Test')
        
    def test_get_success(self, requests_mock):
        acttyp = actiontypes.get(1)
        assert acttyp != None
      
    def test_properties(self, requests_mock):
        acttyp = actiontypes.get(1)
        assert acttyp.type_id == 1
        assert acttyp.name == "Create Sample"
        assert acttyp.object_name == "Sample"
        assert acttyp.admin_only == False
        assert 'ActionType' in repr(acttyp)


import pytest
import io
import tempfile
from sampledbapi import objects
from test import test_authentication, test_users, test_locations

def mock_object():
    return '''{
            "object_id": 1,
            "version_id": 1,
            "action_id": 1,
            "schema": {"title": "Basic Sample Information"},
            "data": {"title": "Basic Sample Information"}
        }'''

def mock_objects():
    return f'[{mock_object()},{mock_object()},{mock_object()}]'

def mock_permission():
    return '"TestPerm"'
    
def mock_permissions():
    return '{"1": "TestPerm", "2": "TestPerm", "3": "TestPerm"}'

def mock_locationOccurence():
    return '''{
            "object_id": 1,
            "location": 1,
            "responsible_user": 1,
            "user": 1,
            "description": "TestDescription",
            "utc_datetime": "2022-11-21T09:39:08.470159"
        }'''
    
def mock_locationOccurences():
    return f'[{mock_locationOccurence()},{mock_locationOccurence()},{mock_locationOccurence()}]'

def mock_file():
    return '''{
            "object_id": 1,
            "file_id": 1,
            "storage": "database",
            "original_file_name": "Test.txt",
            "base64_content": "Test64"
        }'''

def mock_files():
    return f'[{mock_file()},{mock_file()},{mock_file()}]'
    
def mock_comment():
    return '''{
            "object_id": 1,
            "user_id": 1,
            "comment_id": 1,
            "content": "TestContent",
            "utc_datetime": "2022-11-21T09:39:08.470159"
        }'''
    
def mock_comments():
    return f'[{mock_comment()},{mock_comment()},{mock_comment()}]'

class TestObjects():
    
    @pytest.fixture(autouse=True)
    def test_init(self, requests_mock):
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects", text=mock_objects())
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1", text=mock_object())
        requests_mock.post("http://128.176.208.107:8000/api/v1/objects/", text=mock_objects())
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1/versions/1", text=mock_object())
        requests_mock.post("http://128.176.208.107:8000/api/v1/objects/1/versions/")
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1/permissions/public", text=mock_permissions())
        requests_mock.put("http://128.176.208.107:8000/api/v1/objects/1/permissions/public")
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1/permissions/users", text=mock_permissions())
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1/permissions/users/1", text=mock_permission())
        requests_mock.put("http://128.176.208.107:8000/api/v1/objects/1/permissions/users/1")
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1/permissions/groups", text=mock_permissions())
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1/permissions/groups/1", text=mock_permission())
        requests_mock.put("http://128.176.208.107:8000/api/v1/objects/1/permissions/groups/1")
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1/permissions/projects", text=mock_permissions())
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1/permissions/projects/1", text=mock_permission())
        requests_mock.put("http://128.176.208.107:8000/api/v1/objects/1/permissions/projects/1")
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1/locations", text=mock_locationOccurences())
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1/locations/1", text=mock_locationOccurence())
        requests_mock.get("http://128.176.208.107:8000/api/v1/locations/1", text=test_locations.mock_location())
        requests_mock.get("http://128.176.208.107:8000/api/v1/users/1", text=test_users.mock_user())
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1/files", text=mock_files())
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1/files/1", text=mock_file())
        requests_mock.post("http://128.176.208.107:8000/api/v1/objects/1/files/")
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1/comments", text=mock_comments())
        requests_mock.get("http://128.176.208.107:8000/api/v1/objects/1/comments/1", text=mock_comment())
        requests_mock.post("http://128.176.208.107:8000/api/v1/objects/1/comments/")
        
        test_authentication.mock_authenticate(requests_mock)
       
    def test_getList_default(self, requests_mock):
        objs = objects.getList()
        assert len(objs) == 3
        
    def test_getList_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.getList(q=1)
        
    def test_getList_success(self, requests_mock):
        objs = objects.getList(q='Test', action_id=1, action_type='Create', limit=3, offset=3, name_only=True)
        assert len(objs) == 3
        
    def test_get_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get('Test')
            
    def test_get_success(self, requests_mock):
        obj = objects.get(1)
        assert obj != None
      
    def test_properties(self, requests_mock):
        obj = objects.get(1)
        assert obj.object_id == 1
        assert obj.version_id == 1
        assert obj.action_id == 1
        assert len(obj.schema) > 0
        assert len(obj.data) > 0
        assert 'Object' in repr(obj)
        
    def test_create_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.create('Test', {'name': 'Test'})
        
    def test_create_success(self, requests_mock):
        objects.create(1, {'name': 'Test'})
        
    def test_getVersion_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).getVersion('Test')
        
    def test_getVersion_success(self, requests_mock):
        obj = objects.get(1).getVersion(1)
        assert obj != None 
        
    def test_update_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).update('Test')
        
    def test_update_success(self, requests_mock):
        objects.get(1).update({"title": "Basic Sample Information"})
        
    def test_getPublic(self, requests_mock):
        assert objects.get(1).getPublic()
        
    def test_setPublic_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).setPublic('Test')
        
    def test_setPublic_success(self, requests_mock):
        objects.get(1).setPublic(False) 
        
    def test_getAllUsersPermissions(self, requests_mock):
        perm = objects.get(1).getAllUsersPermissions()
        assert type(perm) == dict
        assert len(perm) == 3
        
    def test_getUserPermissions_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).getUserPermissions('Test')
        
    def test_getUserPermissions_success(self, requests_mock):
        perm = objects.get(1).getUserPermissions(1)     
        assert type(perm) == str
        assert perm != None
        
    def test_setUserPermissions_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).setUserPermissions('Test', 'TestPerm')
        
    def test_setUserPermissions_success(self, requests_mock):
        objects.get(1).setUserPermissions(1, 'TestPerm')    
        
    def test_getAllGroupPermissions(self, requests_mock):
        perm = objects.get(1).getAllGroupsPermissions()
        assert type(perm) == dict
        assert len(perm) == 3
        
    def test_getGroupPermissions_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).getGroupPermissions('Test')
        
    def test_getGroupPermissions_success(self, requests_mock):
        perm = objects.get(1).getGroupPermissions(1)     
        assert type(perm) == str
        assert perm != None
        
    def test_setGroupPermissions_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).setGroupPermissions('Test', 'TestPerm')
        
    def test_setGroupPermissions_success(self, requests_mock):
        objects.get(1).setGroupPermissions(1, 'TestPerm')   
    
    def test_getAllProjectGroupPermissions(self, requests_mock):
        perm = objects.get(1).getAllProjectGroupsPermissions()
        assert type(perm) == dict
        assert len(perm) == 3
        
    def test_getProjectGroupPermissions_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).getProjectGroupPermissions('Test')
        
    def test_getProjectGroupPermissions_success(self, requests_mock):
        perm = objects.get(1).getProjectGroupPermissions(1)     
        assert type(perm) == str
        assert perm != None
        
    def test_setProjectGroupPermissions_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).setProjectGroupPermissions('Test', 'TestPerm')
        
    def test_setProjectGroupPermissions_success(self, requests_mock):
        objects.get(1).setProjectGroupPermissions(1, 'TestPerm')  
    
    def test_getLocationOccurences(self, requests_mock):
        locOccs = objects.get(1).getLocationOccurences()
        assert len(locOccs) == 3
        
    def test_getLocationOccurence_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).getLocationOccurence('Test')
            
    def test_getLocationOccurence_success(self, requests_mock):
        locOcc = objects.get(1).getLocationOccurence(1)
        assert locOcc is not None
        
    def test_getLocationOccurence_properties(self, requests_mock):
        locOcc = objects.get(1).getLocationOccurence(1)
        assert locOcc.object_id == 1
        assert locOcc.location is not None
        assert locOcc.responsible_user is not None
        assert locOcc.user is not None
        assert locOcc.description == 'TestDescription'
        assert locOcc.utc_datetime.strftime('%Y-%m-%dT%H:%M:%S.%f') == '2022-11-21T09:39:08.470159'
        assert 'LocationOccurence of object' in repr(locOcc)
    
    def test_getFileList(self, requests_mock):
        files = objects.get(1).getFileList()
        assert len(files) == 3
    
    def test_getFile_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).getFile('Test')
        
    def test_getFile_success(self, requests_mock):
        file = objects.get(1).getFile(1)
        assert file != None
        
    def test_getFile_properties(self, requests_mock):
        file = objects.get(1).getFile(1)
        assert file.object_id == 1
        assert file.file_id == 1
        assert file.storage == 'database'
        assert file.original_file_name == 'Test.txt'
        assert file.base64_content == 'Test64'
        assert 'File' in repr(file)
    
    def test_uploadFile_default(self, requests_mock):
        f = tempfile.NamedTemporaryFile(delete=False)
        f.close()
        objects.get(1).uploadFile(f.name)
        
    def test_uploadFile_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).uploadFile(1)
        
    def test_uploadFile_success(self, requests_mock):
        f = tempfile.NamedTemporaryFile(delete=False)
        f.close()
        objects.get(1).uploadFile(f.name, 'Testfile')
    
    def test_uploadFileRaw_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).uploadFileRaw(1, io.BytesIO(b"some initial binary data: \x00\x01"))
        
    def test_uploadFileRaw_success(self, requests_mock):
        objects.get(1).uploadFileRaw('TestFile', io.BytesIO(b"some initial binary data: \x00\x01"))
    
    def test_postLink_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).postLink(1)
        
    def test_postLink_success(self, requests_mock):
        objects.get(1).postLink('http://128.176.208.107:8000/instruments/1#log_entry-1')
    
    def test_getCommentList(self, requests_mock):
        coms = objects.get(1).getCommentList()
        assert len(coms) == 3
        
    def test_getComment_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).getComment('Test')
        
    def test_getComment_success(self, requests_mock):
        com = objects.get(1).getComment(1)
        assert com != None
    
    def test_getComment_properties(self, requests_mock):
        com = objects.get(1).getComment(1)
        assert com.object_id == 1
        assert com.user_id == 1
        assert com.comment_id == 1
        assert com.content == 'TestContent'
        assert com.utc_datetime.strftime('%Y-%m-%dT%H:%M:%S.%f') == '2022-11-21T09:39:08.470159'
        assert 'Comment on object' in str(com)
      
    def test_postComment_fail(self, requests_mock):
        with pytest.raises(TypeError):
            objects.get(1).postComment(1)
        
    def test_postComment_success(self, requests_mock):
        objects.get(1).postComment('TestComment')
    


import pytest
from sampledbapi import actions
from test import test_authentication

def mock_action():
    return '''{
            "action_id": 1,
            "instrument_id": 2,
            "type": "Sample",
            "type_id": 3,
            "name": "Create",
            "description": "Create",
            "is_hidden": false,
            "schema": {"title": "Basic Sample Information"}
        }'''

def mock_actions():
    return f'[{mock_action()},{mock_action()},{mock_action()}]'

class TestActions():
    
    @pytest.fixture(autouse=True)
    def test_init(self, requests_mock):
        requests_mock.get("http://128.176.208.107:8000/api/v1/actions", text=mock_actions())
        requests_mock.get("http://128.176.208.107:8000/api/v1/actions/1", text=mock_action())
        test_authentication.mock_authenticate(requests_mock)
        
    def test_getList(self, requests_mock):
        acts = actions.getList()
        assert len(acts) == 3
        
    def test_get_fail(self, requests_mock):
        with pytest.raises(TypeError):
            actions.get('Test')
                
    def test_get_success(self, requests_mock):
        act = actions.get(1)
        assert act != None
      
    def test_properties(self, requests_mock):
        act = actions.get(1)
        assert act.action_id == 1
        assert act.instrument_id == 2
        assert act.type == "Sample"
        assert act.type_id == 3
        assert act.name == "Create"
        assert act.description == "Create"
        assert act.is_hidden == False
        assert len(act.schema) > 0
        assert 'Action' in repr(act)


from __future__ import annotations

import base64
import os
from datetime import datetime
from io import IOBase
from typing import BinaryIO, Dict, List, Optional, Any

from requests import Response

from sampledbapi import SampleDBObject, getData, locations, users, postData, putData
from sampledbapi.users import User

__all__ = ["Object", "File", "Comment", "getList", "get",
           "create"]


class Object(SampleDBObject):

    object_id: Optional[int] = None
    version_id: Optional[int] = None
    action_id: Optional[int] = None
    schema: Optional[dict] = None
    data: Optional[dict] = None

    def __repr__(self) -> str:
        return f"Object {self.object_id}"

    def getVersion(self, version_id: int) -> Object:
        """Get the specific version (version_id).

        Args:
            version_id (int): ID of the version to be retrieved.

        Returns:
            Object: Requested :class:`~sampledbapi.objects.Object`.
        """
        if isinstance(version_id, int):
            return Object(getData(
                f"objects/{self.object_id}/versions/{version_id}"
            ))
        else:
            raise TypeError()

    def update(self, data: dict) -> Response:
        """Create a new version.

        The data is a dictionary that has to be formatted according to the
        action's schema. Exemplary data:

        .. code-block::

            {"name": {
                "_type": "text",
                "text": "Example Object"
            }}

        """
        if isinstance(data, dict):
            return postData(f"objects/{self.object_id}/versions/",
                            {"data": data})
        else:
            raise TypeError()

    def getPublic(self) -> bool:
        """Get whether or not the object is public.

        Returns:
            bool: Whether the object is public or not.
        """
        return getData(f"objects/{self.object_id}/permissions/public")

    def setPublic(self, public: bool) -> Response:
        """Set whether or not the object is public.

        Args:
            public (bool): Whether the object is public or not.

        Returns:
            HTTPResponse: `See here. <https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/developer_guide/api.html#object-permissions>`__
        """
        if isinstance(public, bool):
            return putData(f"objects/{self.object_id}/permissions/public",
                           public)
        else:
            raise TypeError()

    def getAllUsersPermissions(self) -> Dict:
        """Get a mapping of user IDs to their permissions.

        Returns:
            Dict: Mapping of user IDs to permissions.
        """
        return getData(f"objects/{self.object_id}/permissions/users")

    def getUserPermissions(self, user_id: int) -> str:
        """Get the permissions of a user.

        Args:
            user_id (int): ID of the user.

        Returns:
            str: Permissions of user for the object.
        """
        if isinstance(user_id, int):
            return getData(
                f"objects/{self.object_id}/permissions/users/{user_id}"
            )
        else:
            raise TypeError()

    def setUserPermissions(self, user_id: int, permissions: str) -> Response:
        """Set the permissions of a user.

        Args:
            user_id (int): ID of the user.
            permissions (str): Permissions of user for the object.

        Returns:
            HTTPResponse: `See here. <https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/developer_guide/api.html#object-permissions>`__
        """
        if isinstance(user_id, int) and isinstance(permissions, str):
            return putData(
                f"objects/{self.object_id}/permissions/users/{user_id}",
                permissions
            )
        else:
            raise TypeError()

    def getAllGroupsPermissions(self) -> Dict:
        """Get a mapping of basic group IDs to their permissions.

        Returns:
            Dict: Mapping of group IDs to permissions.
        """
        return getData(f"objects/{self.object_id}/permissions/groups")

    def getGroupPermissions(self, group_id: int) -> str:
        """Get the permissions of a basic group.

        Args:
            group_id (int): ID of the group.

        Returns:
            str: Permissions of group for the object.
        """
        if isinstance(group_id, int):
            return getData(
                f"objects/{self.object_id}/permissions/groups/{group_id}"
            )
        else:
            raise TypeError()

    def setGroupPermissions(self, group_id: int, permissions: str) -> Response:
        """Set the permissions of a basic group.

        Args:
            group_id (int): ID of the group.
            permissions (str): Permissions of group for the object.

        Returns:
            HTTPResponse: `See here. <https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/developer_guide/api.html#object-permissions>`__
        """
        if isinstance(group_id, int) and isinstance(permissions, str):
            return putData(
                f"objects/{self.object_id}/permissions/groups/{group_id}",
                permissions
            )
        else:
            raise TypeError()

    def getAllProjectGroupsPermissions(self) -> Dict:
        """Get a mapping of project group IDs to their permissions.

        Returns:
            Dict: Mapping of project group IDs to permissions.
        """
        return getData(f"objects/{self.object_id}/permissions/projects")

    def getProjectGroupPermissions(self, project_id: int) -> str:
        """Get the permissions of a project group.

        Args:
            project_id (int): ID of the project group.

        Returns:
            str: Permissions of project group for the object.
        """
        if isinstance(project_id, int):
            return getData(
                f"objects/{self.object_id}/permissions/projects/{project_id}"
            )
        else:
            raise TypeError()

    def setProjectGroupPermissions(self, project_id: int,
                                   permissions: str) -> Response:
        """Set the permissions of a project group.

        Args:
            project_id (int): ID of the project group.
            permissions (str): Permissions of project group for the object.

        Returns:
            HTTPResponse: `See here. <https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/developer_guide/api.html#object-permissions>`__
        """
        if isinstance(project_id, int) and isinstance(permissions, str):
            return putData(
                f"objects/{self.object_id}/permissions/projects/{project_id}",
                permissions
            )
        else:
            raise TypeError()
            
    def getLocationOccurences(self) -> List[LocationOccurence]:
        """
        Get a list of all object locations assignments for a specific object.

        Args:

        Returns:
            List: `See here. <https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/developer_guide/api.html#reading-a-list-of-an-object-s-locations>`__

        """
        return [LocationOccurence(i) for i in getData(f"objects/{self.object_id}/locations")]
        
    def getLocationOccurence(self, location_id: int) -> LocationOccurence:
        """
        Get a specific object location assignment (index) for a specific object.

        Args:
            location_id (int) : ID of the location

        Returns:
            LocationOccurence: `See here. <https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/developer_guide/api.html#reading-an-object-s-location>`__


        """
        if isinstance(location_id, int):
            return LocationOccurence(getData(
                f"objects/{self.object_id}/locations/{location_id}"
            ))
        else:
            raise TypeError()

    def getFileList(self) -> List:
        """Get a list of all files.

        Returns:
            List: `See here. <https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/developer_guide/api.html#files>`__
        """
        return [File(i) for i in getData(f"objects/{self.object_id}/files")]

    def getFile(self, file_id: int) -> File:
        """Get a specific file (file_id).

        Args:
            file_id (int): ID of the file.

        Returns:
            Dict: `See here. <https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/developer_guide/api.html#files>`__
        """
        if isinstance(file_id, int):
            return File(getData(
                f"objects/{self.object_id}/files/{file_id}"
            ))
        else:
            raise TypeError()

    def uploadFile(self, path: str, name: Optional[str] = None) -> Response:
        """Create a new file with local storage.

        Args:
            path (str): Path of the file to be uploaded.

        Returns:
            HTTPResponse: `See here. <https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/developer_guide/api.html#files>`__
        """
        if isinstance(path, str):
            with open(path, "rb") as f:
                f.name
                r = self.uploadFileRaw(f.name if name is None else name, f)
            return r
        else:
            raise TypeError()

    def uploadFileRaw(self, name: str, file_obj: BinaryIO) -> Response:
        """Create a new file with local storage.

        Args:
            name (str): Name that the file will have online.
            file_obj (BinaryIO): A binary stream that can be read to be uploaded.

        Returns:
            HTTPResponse: `See here. <https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/developer_guide/api.html#files>`__
        """
        if isinstance(name, str) and isinstance(file_obj, IOBase):
            base64encoded = base64.b64encode(file_obj.read())
            return postData(f"objects/{self.object_id}/files/",
                            {"storage": "local", "original_file_name": name,
                             "base64_content": base64encoded.decode()})
        else:
            raise TypeError()

    def postLink(self, url: str) -> Response:
        """Create a new file with url storage.

        Args:
            url (str): URL to be stored.

        Returns:
            HTTPResponse: `See here. <https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/developer_guide/api.html#files>`__
        """
        if isinstance(url, str):
            return postData(f"objects/{self.object_id}/files/",
                            {"storage": "url", "url": url})
        else:
            raise TypeError()

    def getCommentList(self) -> List[Comment]:
        """Get a list of all comments.

        Returns:
            List: List of :class:`~sampledbapi.objects.Comment`.
        """
        return [Comment(c)
                for c in getData(f"objects/{self.object_id}/comments")]

    def getComment(self, comment_id: int) -> Comment:
        """Get specific comment (comment_id).

        Args:
            comment_id (int): ID of the comment.

        Returns:
            Object: Requested :class:`~sampledbapi.objects.Comment`.
        """
        if isinstance(comment_id, int):
            return Comment(getData(
                    f"objects/{self.object_id}/comments/{comment_id}"))
        else:
            raise TypeError()

    def postComment(self, comment: str) -> Response:
        """Create a new comment.

        Args:
            comment (str): Comment to be posted.

        Returns:
            HTTPResponse: `See here. <https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/developer_guide/api.html#comments>`__
        """
        if isinstance(comment, str):
            return postData(f"objects/{self.object_id}/comments/",
                            {"content": comment})
        else:
            raise TypeError()

def getList(q: str = "", action_id: int = -1, action_type: str = "",
            limit: int = -1, offset: int = -1,
            name_only: bool = False) -> List[Object]:
    """Get a list of all objects visible to the current user.

    The list only contains the current version of each object. By passing the
    parameter q to the query, the Advanced Search can be used. By passing the
    parameters action_id or action_type objects can be filtered by the action
    they were created with or by their type (e.g. sample or measurement).

    Instead of returning all objects, the parameters limit and offset can be
    used to reduce to maximum number of objects returned and to provide an
    offset in the returned set, so allow simple pagination.

    If the parameter name_only is provided, the object data and schema will be
    reduced to the name property, omitting all other properties and schema
    information.

    Args:
        q (str): Search string for advanced search, `see here. <https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/user_guide/objects.html#advanced-search>`__
        action_id (int): Filter by action ID.
        action_type (str): Filter by action type.
        limit (int): Limit number of results (helpful for pagination).
        offset (int): Offset for limited retrieval of results (pagination).
        name_only (bool): Only names will be returned, no other information.

    Returns:
        List: List of :class:`~sampledbapi.objects.Object`.
    """
    if (isinstance(q, str) and isinstance(action_id, int) and
            isinstance(action_type, str) and isinstance(limit, int) and
            isinstance(offset, int) and isinstance(name_only, bool)):
        s = "objects"
        pars : Dict[str, Any] = {}
        if q != "":
            pars["q"] = q
        if action_id > 0:
            pars["action_id"] = action_id
        if action_type != "":
            pars["action_type"] = action_type
        if limit > 0:
            pars["limit"] = limit
        if offset > 0:
            pars["offset"] = offset
        if name_only:
            pars["name_only"] = "true"

        if len(pars) > 0:
            s += "?"
        for i, p in enumerate(pars):
            s += f"{p}={pars[p]}"
            if i < len(pars) - 1:
                s += "&"

        return [Object(o) for o in getData(s)]
    else:
        raise TypeError()


def get(object_id: int) -> Object:
    """Get the current version of an object (object_id).

    Args:
        object_id (int): ID of the object.

    Returns:
        Object: Requested :class:`~sampledbapi.objects.Object`.
    """
    if isinstance(object_id, int):
        return Object(getData(f"objects/{object_id}"))
    else:
        raise TypeError()


def create(action_id: int, data: dict) -> Response:
    """Create a new object.

    The data is a dictionary that has to be formatted according to the action's
    schema. Exemplary data:

    .. code-block::

        {"name": {
            "_type": "text",
            "text": "Example Object"
        }}

    """
    if isinstance(action_id, int) and isinstance(data, dict):
        return postData("objects/", {"action_id": action_id, "data": data})
    else:
        raise TypeError()

class File(SampleDBObject):
    
    object_id:  Optional[int] = None
    file_id:  Optional[int] = None
    storage:  Optional[str] = None
    original_file_name:  Optional[str] = None
    base64_content:  Optional[str] = None
    
    def __repr__(self) -> str:
        return f"File {self.file_id}, " \
            + f"Name {self.original_file_name}"
            
class LocationOccurence(SampleDBObject):

    object_id: Optional[int] = None
    location: Optional[locations.Location] = None
    responsible_user: Optional[User] = None
    user: Optional[User] = None
    description: Optional[str] = None
    utc_datetime: Optional[datetime] = None
    
    def __init__(self, d: Dict):
        """Initialize a new instrument from dictionary."""
        super().__init__(d)
        if "location" in d:
            self.location = locations.get(d["location"])
        if "responsible_user" in d:
            self.responsible_user = users.get(d["responsible_user"])
        if "user" in d:
            self.user = users.get(d["user"])
        if "utc_datetime" in d:
            self.utc_datetime = datetime.strptime(d["utc_datetime"], '%Y-%m-%dT%H:%M:%S.%f')

    def __repr__(self) -> str:
        return f"LocationOccurence of object {self.object_id} " \
            + "(at {self.location.name})"

class Comment(SampleDBObject):

    object_id:  Optional[int] = None
    user_id:  Optional[int] = None
    comment_id:  Optional[int] = None
    content:  Optional[str] = None
    utc_datetime:  Optional[datetime] = None

    def __init__(self, d: Dict):
        """Initialize a new comment from dictionary."""
        super().__init__(d)
        if "utc_datetime" in d:
            self.utc_datetime = datetime.strptime(d["utc_datetime"], '%Y-%m-%dT%H:%M:%S.%f')

    def __str__(self) -> str:
        return f"Comment on object {self.object_id}, " \
            + f"posted {self.utc_datetime}"



import pytest
from sampledbapi import users
from test import test_authentication

def mock_user():
    return '''{
            "user_id": 1,
            "name": "Nils Weber",
            "orcid": "12345",
            "affiliation": "WWU Münster"
        }'''

def mock_users():
    return f'[{mock_user()},{mock_user()},{mock_user()}]'

class TestUsers():
    
    @pytest.fixture(autouse=True)
    def test_init(self, requests_mock):
        requests_mock.get("http://128.176.208.107:8000/api/v1/users", text=mock_users())
        requests_mock.get("http://128.176.208.107:8000/api/v1/users/1", text=mock_user())
        requests_mock.get("http://128.176.208.107:8000/api/v1/users/me", text=mock_user())
        test_authentication.mock_authenticate(requests_mock)
        
    def test_getList(self, requests_mock):
        usrs = users.getList()
        assert len(usrs) == 3
        
    def test_get_fail(self, requests_mock):
        with pytest.raises(TypeError):
            users.get('Test')
            
    def test_get_success(self, requests_mock):
        usr = users.get(1)
        assert usr != None
      
    def test_properties(self, requests_mock):
        usr = users.get(1)
        assert usr.user_id == 1
        assert usr.name == "Nils Weber"
        assert usr.orcid == "12345"
        assert usr.affiliation == "WWU Münster"
        assert 'User' in repr(usr)
        
    def test_getCurrent(self, requests_mock):
        usr = users.getCurrent()
        assert usr != None


from __future__ import annotations

import base64
import os
from datetime import datetime
from typing import Dict, List, Optional, Any

from requests import Response

from sampledbapi import SampleDBObject, getData, postData, users

__all__ = ["Instrument", "InstrumentLogCategory", "InstrumentLogEntry",
           "InstrumentLogFileAttachment", "InstrumentLogObjectAttachment",
           "getList", "get"]


class Instrument(SampleDBObject):

    instrument_id: Optional[int] = None
    name: Optional[str] = None
    description: Optional[str] = None
    is_hidden: Optional[bool] = None
    instrument_scientists: Optional[List[users.User]] = None

    def __init__(self, d: Dict):
        """Initialize a new instrument from dictionary."""
        super().__init__(d)
        if "instrument_scientists" in d:
            self.instrument_scientists = [
                users.get(i) for i in d["instrument_scientists"]]

    def __repr__(self) -> str:
        return f"Instrument {self.instrument_id} ({self.name})"

    def getLogEntryList(self) -> List[InstrumentLogEntry]:
        """Get a list of all log entries.

        Returns:
            List: List of :class:`~sampledbapi.instrument.InstrumentLogEntry`.
        """
        if self.instrument_id is not None:
            return [InstrumentLogEntry(self.instrument_id, log) for log in getData(
                f"instruments/{self.instrument_id}/log_entries")]
        else:
            raise Exception('instrument_id can not be None')

    def getLogEntry(self, log_entry_id: int) -> InstrumentLogEntry:
        """Get the specific log entry (log_entry_id).

        Args:
            log_entry_id (int): ID of the specific log entry.

        Returns:
            InstrumentLogEntry: The requested
                :class:`~sampledbapi.instrument.InstrumentLogEntry`.
        """
        if isinstance(log_entry_id, int):
            if self.instrument_id is not None:
                return InstrumentLogEntry(self.instrument_id, getData(
                    f"instruments/{self.instrument_id}/log_entries/{log_entry_id}"
                ))
            else:
                raise Exception('instrument_id can not be None')
        else:
            raise TypeError()

    def getLogCategoryList(self) -> List[InstrumentLogCategory]:
        """Get a list of all log categories.

        Returns:
            List: List of :class:`~sampledbapi.instrument.InstrumentLogCategory`.
        """
        return [InstrumentLogCategory(c) for c in getData(
            f"instruments/{self.instrument_id}/log_categories")]

    def getLogCategory(self, log_category_id: int) -> InstrumentLogCategory:
        """Get a specific log category (log_category_id).

        Args:
            log_category_id (int): ID of the specific log category.

        Returns:
            InstrumentLogCategory: The requested
                :class:`~sampledbapi.instrument.InstrumentLogCategory`.
        """
        if isinstance(log_category_id, int):
            return InstrumentLogCategory(getData(
                f"instruments/{self.instrument_id}/log_categories/" +
                f"{log_category_id}"))
        else:
            raise TypeError()

    def createLogEntry(self, content: str, category_ids: List = [],
                       file_attachments: List = [],
                       object_attachments: List = []) -> Response:
        """Create a log entry for an instrument (instrument_id) and optionally attach files and objects to it.

        Args:
            content (str): Log message.
            category_ids (list of ints): An optional list of category IDs for the log entry.
            file_attachments (list of strings): List of file paths to be read and attached to the log entry.
            object_attachments (list of integers): Object IDs to be attached to the log entry.

        Returns:
            HTTPResponse: `See here. <https://scientific-it-systems.iffgit.fz-juelich.de/SampleDB/developer_guide/api.html#instrument-log-entries>`_
        """
        if (isinstance(content, str) and isinstance(category_ids, list) and
                isinstance(file_attachments, list) and
                isinstance(object_attachments, list)):
            data : Dict[str, Any] = {"content": content}

            def conv_file(path: str):
                with open(path, "rb") as f:
                    base64encoded = base64.b64encode(f.read())
                return {"file_name": os.path.basename(path),
                        "base64_content": base64encoded.decode()}

            if len(category_ids) > 0:
                data["category_ids"] = category_ids
            if len(file_attachments) > 0:
                data["file_attachments"] = [
                    conv_file(f) for f in file_attachments]
            if len(object_attachments) > 0:
                data["object_attachments"] = [
                    {"object_id": i} for i in object_attachments]

            return postData(
                f"instruments/{self.instrument_id}/log_entries/", data)
        else:
            raise TypeError()


def getList() -> List[Instrument]:
    """Get a list of all instruments.

    Returns:
        List: List of :class:`~sampledbapi.instrument.Instrument` objects.
    """
    return [Instrument(i) for i in getData("instruments")]


def get(instrument_id: int) -> Instrument:
    """Get the specific instrument (instrument_id).

    Args:
        instrument_id (int): ID of the specific instrument.

    Returns:
        Instrument: The requested :class:`~sampledbapi.instrument.Instrument`.
    """
    if isinstance(instrument_id, int):
        return Instrument(getData(f"instruments/{instrument_id}"))
    else:
        raise TypeError()


"""Instrument log entries"""


class InstrumentLogCategory(SampleDBObject):

    category_id: Optional[int] = None
    title: Optional[str] = None

    def __repr__(self) -> str:
        return f"InstrumentLogCategory {self.category_id} ({self.title})"


class InstrumentLogEntry(SampleDBObject):

    log_entry_id: Optional[int] = None
    instrument_id: Optional[int] = None
    utc_datetime: Optional[datetime] = None
    author: Optional[users.User] = None
    content: Optional[str] = None
    categories: Optional[List[InstrumentLogCategory]] = None

    def __init__(self, instrument_id: int, d: Dict):
        """Initialize a new instrument log entry from dictionary."""
        super().__init__(d)
        self.instrument_id = instrument_id
        if "categories" in d:
            self.categories = [
                InstrumentLogCategory(c) for c in d["categories"]]
        if "author" in d:
            self.author = users.get(d["author"])
        if "utc_datetime" in d:
            self.utc_datetime = datetime.strptime(d["utc_datetime"], '%Y-%m-%dT%H:%M:%S.%f')

    def __repr__(self) -> str:
        return f"InstrumentLogEntry {self.log_entry_id} " \
            + f"(created {self.utc_datetime})"

    def getFileAttachmentList(self) -> List[InstrumentLogFileAttachment]:
        """Get a list of file attachments.

        Returns:
            List: List of
                :class:`~sampledbapi.instrument.InstrumentLogFileAttachment`.
        """
        return [InstrumentLogFileAttachment(f) for f in getData(
            f"instruments/{self.instrument_id}/log_entries/"
            f"{self.log_entry_id}/file_attachments")]

    def getFileAttachment(self, file_attachment_id: int
                          ) -> InstrumentLogFileAttachment:
        """Get a specific file attachment (file_attachment_id).

        Args:
            file_attachment_id (int): ID of the file attachment.

        Returns:
            InstrumentLogFileAttachment: The requested
                :class:`~sampledbapi.instrument.InstrumentLogFileAttachment`.
        """
        if isinstance(file_attachment_id, int):
            return InstrumentLogFileAttachment(getData(
                f"instruments/{self.instrument_id}/log_entries/"
                + f"{self.log_entry_id}/file_attachments/{file_attachment_id}"))
        else:
            raise TypeError()

    def getObjectAttachmentList(self) -> List[InstrumentLogObjectAttachment]:
        """Get a list of object attachments.

        Returns:
            List: List of
                :class:`~sampledbapi.instrument.InstrumentLogObjectAttachment`.
        """
        return [InstrumentLogObjectAttachment(o) for o in getData(
            f"instruments/{self.instrument_id}/log_entries/"
            f"{self.log_entry_id}/object_attachments")]

    def getObjectAttachment(self, object_attachment_id: int) -> InstrumentLogObjectAttachment:
        """Get a specific object attachment (object_attachment_id).

        Args:
            object_attachment_id (int): ID of the object attachment.

        Returns:
            InstrumentLogObjectAttachment: The requested
                :class:`~sampledbapi.instrument.InstrumentLogObjectAttachment`.
        """
        if isinstance(object_attachment_id, int):
            return InstrumentLogObjectAttachment(getData(
                f"instruments/{self.instrument_id}/log_entries/"
                f"{self.log_entry_id}/object_attachments/"
                f"{object_attachment_id}"))
        else:
            raise TypeError()


class InstrumentLogFileAttachment(SampleDBObject):

    file_attachment_id: Optional[int] = None
    file_name: Optional[str] = None
    content: Optional[str] = None

    def __repr__(self) -> str:
        return f"InstrumentLogFileAttachment {self.file_attachment_id} " \
            + f"({self.file_name})"


class InstrumentLogObjectAttachment(SampleDBObject):

    object_attachment_id: Optional[int] = None
    object_id: Optional[int] = None

    def __repr__(self) -> str:
        return f"InstrumentLogObjectAttachment {self.object_attachment_id} " \
            + f"(for object {self.object_id})"



import pytest
from sampledbapi import locations
from test import test_authentication

def mock_location():
    return '''{
            "location_id": 1,
            "name": "IG1",
            "description": "IG1",
            "parent_location_id": 2,
            "type_id": 3,
            "is_hidden": false
        }'''

def mock_locations():
    return f'[{mock_location()},{mock_location()},{mock_location()}]'

class TestLocations():
    
    @pytest.fixture(autouse=True)
    def test_init(self, requests_mock):
        requests_mock.get("http://128.176.208.107:8000/api/v1/locations", text=mock_locations())
        requests_mock.get("http://128.176.208.107:8000/api/v1/locations/1", text=mock_location())
        test_authentication.mock_authenticate(requests_mock)
        
    def test_getList(self, requests_mock):
        locs = locations.getList()
        assert len(locs) == 3
        
    def test_get_fail(self, requests_mock):
        with pytest.raises(TypeError):
            locations.get('Test')
            
    def test_get_success(self, requests_mock):
        loc = locations.get(1)
        assert loc != None
      
    def test_properties(self, requests_mock):
        loc = locations.get(1)
        assert loc.location_id == 1
        assert loc.name == "IG1"
        assert loc.description == "IG1"
        assert loc.parent_location_id == 2
        assert loc.type_id == 3
        assert loc.is_hidden == False
        assert 'Location' in repr(loc)


import pytest
from sampledbapi import locationtypes
from test import test_authentication

def mock_locationtype():
    return '''{
            "location_type_id": 1,
            "name": "Room"
        }'''

def mock_locationtypes():
    return f'[{mock_locationtype()},{mock_locationtype()},{mock_locationtype()}]'

class TestLocationTypes():
    
    @pytest.fixture(autouse=True)
    def test_init(self, requests_mock):
        requests_mock.get("http://128.176.208.107:8000/api/v1/location_types", text=mock_locationtypes())
        requests_mock.get("http://128.176.208.107:8000/api/v1/location_types/1", text=mock_locationtype())
        test_authentication.mock_authenticate(requests_mock)
    
    def test_getList(self):
        loctyps = locationtypes.getList()
        assert len(loctyps) == 3
    
    def test_get_fail(self, requests_mock):
        with pytest.raises(TypeError):
            locationtypes.get('Test')
            
    def test_get_success(self):
        loctyp = locationtypes.get(1)
        assert loctyp != None
        
    def test_properties(self):
        loctyp = locationtypes.get(1)
        assert loctyp.location_type_id == 1
        assert loctyp.name == 'Room'
        assert 'LocationType' in repr(loctyp)


import os
from codecs import open  # To use a consistent encoding

from setuptools import find_packages, setup  # Prefer setuptools over distutils

here = os.path.abspath(os.path.dirname(__file__))

# Get the long description from the relevant file
with open(os.path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

# with open("requirements.txt") as f:
#     requirements = f.read().splitlines()


out = setup(
    name="sampledbapi",

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # http://packaging.python.org/en/latest/tutorial.html#version
    version="0.5.0",

    description="API wrapper for SampleDB",
    long_description=long_description,
    long_description_content_type="text/markdown",

    # The project's main homepage.
    url="https://zivgitlab.uni-muenster.de/ag-salinga/sampledb-api-wrapper",

    # Author details
    author="AG Salinga, WWU Münster",
    author_email="itsaling@uni-muenster.de",

    # Choose your license
    license="MIT",

    # See https://PyPI.python.org/PyPI?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 4 - Beta",

        # Indicate who your project is intended for
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering",

        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: MIT License",

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],

    # What does your project relate to?
    keywords="api sampledb",

    packages=["sampledbapi"],

    package_dir={"sampledbapi": "./sampledbapi"},

    package_data={"": []},

    data_files=[],

    # Requirements
    # install_requires=requirements,

    # Entry point (none so far)
    # entry_points={
    #     "console_scripts": [
    #         "puzzlestream = puzzlestream.launch:launchPuzzlestream",
    #     ]
    # },
)



from typing import List, Optional

from sampledbapi import SampleDBObject, getData
from sampledbapi.actions import Action

__all__ = ["ActionType", "getList", "get"]


class ActionType(SampleDBObject):

    type_id: Optional[int] = None
    name: Optional[str] = None
    object_name: Optional[str] = None
    admin_only: Optional[bool] = None

    def __repr__(self) -> str:
        return f"ActionType {self.type_id} ({self.name})"


def getList() -> List[ActionType]:
    """Get a list of all action types.

    Returns:
        List: List of :class:`~sampledbapi.actiontypes.Actiontype` objects.
    """
    return [ActionType(a) for a in getData("action_types")]


def get(type_id: int) -> ActionType:
    """Get the specific action type (type_id).

    Args:
        type_id (int): ID of the action type.

    Returns:
        ActionType: The requested :class:`~sampledbapi.actiontypes.ActionType`.
    """
    if isinstance(type_id, int):
        return ActionType(getData(f"action_types/{type_id}"))
    else:
        raise TypeError()



import pytest
import tempfile
from sampledbapi import instruments
from test import test_authentication, test_users

def mock_instrument():
    return '''{
            "instrument_id": 1,
            "name": "MBE",
            "description": "Molecular-beam epitaxy",
            "is_hidden": false,
            "instrument_scientists": [1]
        }'''

def mock_instruments():
    return f'[{mock_instrument()},{mock_instrument()},{mock_instrument()}]'

def mock_logEntry():
    return '''{
            "log_entry_id": 1,
            "instrument_id": 1,
            "utc_datetime": "2022-11-21T09:39:08.470159",
            "author": 1,
            "content": "Dies ist ein Test-Text",
            "categories": ["1"]
        }'''
    
    #"categories": [1] ?

def mock_logEntries():
    return f'[{mock_logEntry()},{mock_logEntry()},{mock_logEntry()}]'

def mock_logCategory():
    return '''{
            "category_id": 1,
            "title": "TestCategory"
        }'''

def mock_logCategories():
    return f'[{mock_logCategory()},{mock_logCategory()},{mock_logCategory()}]'

def mock_fileAttachment():
    return '''{
            "file_attachment_id": 1,
            "file_name": "TestName",
            "content": "TestContent"
        }'''
    
def mock_fileAttachments():
    return f'[{mock_fileAttachment()},{mock_fileAttachment()},{mock_fileAttachment()}]'
    
def mock_objectAttachment():
    return '''{
            "object_attachment_id": 1,
            "object_id": 1
        }'''
    
def mock_objectAttachments():
    return f'[{mock_objectAttachment()},{mock_objectAttachment()},{mock_objectAttachment()}]'

class TestInstruments():
    
    @pytest.fixture(autouse=True)
    def test_init(self, requests_mock):
        requests_mock.get("http://128.176.208.107:8000/api/v1/users/1", text=test_users.mock_user())
        requests_mock.get("http://128.176.208.107:8000/api/v1/instruments", text=mock_instruments())
        requests_mock.get("http://128.176.208.107:8000/api/v1/instruments/1", text=mock_instrument())
        requests_mock.get("http://128.176.208.107:8000/api/v1/instruments/1/log_entries", text=mock_logEntries())
        requests_mock.get("http://128.176.208.107:8000/api/v1/instruments/1/log_entries/1", text=mock_logEntry())
        requests_mock.get("http://128.176.208.107:8000/api/v1/instruments/1/log_categories", text=mock_logCategories())
        requests_mock.get("http://128.176.208.107:8000/api/v1/instruments/1/log_categories/1", text=mock_logCategory())
        requests_mock.get("http://128.176.208.107:8000/api/v1/instruments/1/log_entries/1/file_attachments", text=mock_fileAttachments())
        requests_mock.get("http://128.176.208.107:8000/api/v1/instruments/1/log_entries/1/file_attachments/1", text=mock_fileAttachment())
        requests_mock.get("http://128.176.208.107:8000/api/v1/instruments/1/log_entries/1/object_attachments", text=mock_objectAttachments())
        requests_mock.get("http://128.176.208.107:8000/api/v1/instruments/1/log_entries/1/object_attachments/1", text=mock_objectAttachment())
        requests_mock.post("http://128.176.208.107:8000/api/v1/instruments/1/log_entries/")

        test_authentication.mock_authenticate(requests_mock)
     
    def test_getList(self, requests_mock):
        insts = instruments.getList()
        assert len(insts) == 3
        
    def test_get_fail(self, requests_mock):
        with pytest.raises(TypeError):
            instruments.get('Test')
        
    def test_get_success(self, requests_mock):
        inst = instruments.get(1)
        assert inst != None
      
    def test_properties(self, requests_mock):
        inst = instruments.get(1)
        assert inst.instrument_id == 1
        assert inst.name == "MBE"
        assert inst.description == "Molecular-beam epitaxy"
        assert inst.is_hidden == False
        assert inst.instrument_scientists != None
        assert 'Instrument' in repr(inst)
        
    def test_getLogEntryList(self, requests_mock):
        logs = instruments.get(1).getLogEntryList()
        assert len(logs) == 3
        
    def test_getLogEntry_fail(self, requests_mock):
        with pytest.raises(TypeError):
            instruments.get(1).getLogEntry('Test')
        
    def test_getLogEntry_success(self, requests_mock):
        log = instruments.get(1).getLogEntry(1)
        assert log != None
        
    def test_logEntry_properties(self, requests_mock):
        log = instruments.get(1).getLogEntry(1)
        assert log.log_entry_id == 1
        assert log.instrument_id == 1
        assert log.utc_datetime.strftime('%Y-%m-%dT%H:%M:%S.%f') == '2022-11-21T09:39:08.470159'
        assert log.author != None
        assert log.content == 'Dies ist ein Test-Text'
        assert len(log.categories) > 0
        assert 'InstrumentLogEntry' in repr(log)
        
    def test_getLogCategoryList(self, requests_mock):
        logCats = instruments.get(1).getLogCategoryList()
        assert len(logCats) == 3
        
    def test_getLogCategory_fail(self, requests_mock):
        with pytest.raises(TypeError):
            instruments.get(1).getLogCategory('Test')
            
    def test_getLogCategory_success(self, requests_mock):
        logCat = instruments.get(1).getLogCategory(1)
        assert logCat != None
        
    def test_getLogCategory_properties(self, requests_mock):
        logCat = instruments.get(1).getLogCategory(1)
        assert logCat.category_id == 1
        assert logCat.title == 'TestCategory'
        assert 'InstrumentLogCategory' in repr(logCat)
        
    def test_createLogEntry_fail(self, requests_mock):
        with pytest.raises(TypeError):
            instruments.get(1).createLogEntry(1, [], [], [])
        
    def test_createLogEntry_success(self, requests_mock):
        new_file, filename = tempfile.mkstemp()
        instruments.get(1).createLogEntry('TestContent', [1], [filename], [1])
    
    def test_getFileAttachmentList(self, requests_mock):
        fileAtts = instruments.get(1).getLogEntry(1).getFileAttachmentList()
        assert len(fileAtts) == 3
        
    def test_getFileAttachment_fail(self, requests_mock):
        with pytest.raises(TypeError):
            instruments.get(1).getLogEntry(1).getFileAttachment('Test')
        
    def test_getFileAttachment_success(self, requests_mock):
        fileAtt = instruments.get(1).getLogEntry(1).getFileAttachment(1)
        assert fileAtt != None
        
    def test_getFileAttachment_properties(self, requests_mock):
        fileAtt = instruments.get(1).getLogEntry(1).getFileAttachment(1)
        assert fileAtt.file_attachment_id == 1
        assert fileAtt.file_name == 'TestName'
        assert fileAtt.content == 'TestContent'
        assert 'InstrumentLogFileAttachment' in repr(fileAtt)
    
    def test_getObjectAttachmentList(self, requests_mock):
        objAtts = instruments.get(1).getLogEntry(1).getObjectAttachmentList()
        assert len(objAtts) == 3
        
    def test_getObjectAttachment_fail(self, requests_mock):
        with pytest.raises(TypeError):
            instruments.get(1).getLogEntry(1).getObjectAttachment('Test')
            
    def test_getObjectAttachment_success(self, requests_mock):
        objAtt = instruments.get(1).getLogEntry(1).getObjectAttachment(1)
        assert objAtt != None
        
    def test_getObjectAttachment_properties(self, requests_mock):
        objAtt = instruments.get(1).getLogEntry(1).getObjectAttachment(1)
        assert objAtt.object_attachment_id == 1
        assert objAtt.object_id == 1
        assert 'InstrumentLogObjectAttachment' in repr(objAtt)


import json
from typing import Any, Dict, List

import requests

__all__ = ["authenticate", "actions", "actiontypes", "instruments",
           "locations", "locationtypes","objects", "users"]

_address = None
_api_key = None


class SampleDBObject:

    def __init__(self, d: Dict):
        """Initialize class attributes from dictionary."""
        for key in dir(self):
            if not key.startswith("__") and key in d:
                setattr(self, key, d[key])


def authenticate(address: str, api_key: str):
    """Authenticate with a SampleDB instance

    To retrieve data from SampleDB and upload data to it, you have to
    authenticate first.

    Arguments:
        address: URL of the SampleDB server.
        api_key: API token you can generate using "Preferences/API Tokens".
    """

    global _address, _api_key
    old_address, old_key = _address, _api_key
    _address, _api_key = address, api_key
    try:
        getData("actions")
    except requests.exceptions.HTTPError as e:
        _address, _api_key = old_address, old_key
        raise Exception("Authentication not successful: " + str(e))


def __headers() -> Dict:
    if _api_key is not None:
        return {"Authorization": "Bearer " + _api_key}
    else:
        raise Exception("You need to provide an API key.")


def getData(path: str) -> Any:
    if _address is not None:
        address = _address + "/api/v1/" + path
        r = requests.get(address, headers=__headers())
        if r.status_code == 200 or r.status_code == 201:
            try:
                return json.loads(r.text)
            except Exception as e:
                # We need better error handling here
                raise Exception("JSON could not be parsed: " + str(e) +
                                "\nJSON was\n" + r.text)
        else:
            r.raise_for_status()
    else:
        raise Exception("You have to authenticate first.")


def postData(path: str, data):
    if _address is not None:
        address = _address + "/api/v1/" + path
        data = json.dumps(data)
        requests.post(address, headers=__headers(), data=data).raise_for_status()
    else:
        raise Exception("You have to authenticate first.")


def putData(path: str, data):
    if _address is not None:
        address = _address + "/api/v1/" + path
        data = json.dumps(data)
        requests.put(address, headers=__headers(), data=data).raise_for_status()
    else:
        raise Exception("You have to authenticate first.")



"""
This is the module docstring.
"""

from typing import List, Optional

from sampledbapi import SampleDBObject, getData

__all__ = ["Action", "getList", "get"]


class Action(SampleDBObject):

    action_id: Optional[int] = None
    instrument_id: Optional[int] = None
    type: Optional[str] = None
    type_id: Optional[int] = None
    name: Optional[str] = None
    description: Optional[str] = None
    is_hidden: Optional[bool] = None
    schema: Optional[dict] = None

    def __repr__(self) -> str:
        return f"Action {self.action_id} ({self.name})"


def getList() -> List[Action]:
    """Get a list of all actions.

    Returns:
        List: List of :class:`~sampledbapi.actions.Action` objects.
    """
    return [Action(a) for a in getData("actions")]


def get(action_id: int) -> Action:
    """Get the specific action (action_id).

    Args:
        action_id (int): ID of the action to be retrieved.

    Returns:
        Action: The requested :class:`~sampledbapi.action.Action`.
    """
    if isinstance(action_id, int):
        return Action(getData(f"actions/{action_id}"))
    else:
        raise TypeError()
